#include <p24fxxxx.h>
#include <ports.h>
#include <timer.h>
#include ".\..\App1632_LCM.h"
#include <adc.h>

/* Configuration Settings */
// OSC Input = XT + PLL
// CPU Clock = ( 96000000 / 3 ) / 1
// Disabld WDT , ICSP/ICD = PGC2 , PGD2 
_CONFIG1( JTAGEN_OFF & GCP_OFF & GWRP_OFF & ICS_PGx2 &
		  FWDTEN_OFF & WINDIS_OFF & FWPSA_PR128 & WDTPS_PS32768 )

_CONFIG2( IESO_OFF & PLLDIV_DIV2 & FNOSC_PRIPLL & 
		  FCKSM_CSDCMD & OSCIOFNC_OFF & IOL1WAY_OFF & DISUVREG_OFF & POSCMOD_XT )

_CONFIG3( WPEND_WPSTARTMEM & WPCFG_WPCFGDIS & WPDIS_WPDIS & WPFP_WPFP511 )

#define SystemFrequency		( ( 96000000L / 3 ) / 1 ) / 2
#define Timer1Tick			( SystemFrequency / 256 / Timer1TogglesPerSec )
#define Timer1TogglesPerSec	10
#define Timer3Tick			( SystemFrequency / 256 / Timer3TogglesPerSec )
#define Timer3TogglesPerSec	10

const unsigned char LCMString1[ ] = "PIC24FJ256GB110 "; 
const unsigned char LCMString2[ ] = "VR1:";

unsigned int VR1Value = 0;
unsigned char ADDown = 0;

void __attribute__( ( interrupt , no_auto_psv ) ) _T1Interrupt( void )
{
	IFS0bits.T1IF = 0;

	mPORTAToggleBits( 0xff );
}

void __attribute__( ( interrupt , no_auto_psv ) )_ADC1Interrupt ( void )
{
	// MADC_XСC
	IFS0bits.AD1IF = 0;

	if( AD1CON2bits.BUFS == 1 )
		VR1Value =  ( unsigned int ) ReadADC10( 0 );
	else
		VR1Value =  ( unsigned int ) ReadADC10( 8 );

	ADDown = 1;
}

int main( void )
{
	// Divide by 1
	CLKDIVbits.CPDIV0 = 0;
	CLKDIVbits.CPDIV1 = 0;
	
	// CPU Clock = PB Clock / 1
	CLKDIVbits.DOZEN = 0;
 
 	// LEDI/O Port]wƦXAëwI/O PortAC
	mPORTAConfig( 0x00 );
	mPORTAWrite( 0x0f );

	LCM_Init( );

	LCM_SetCursor( 0 , 0 );
	LCM_PutROMString( LCMString1 );
	LCM_SetCursor( 0 , 1 );
	LCM_PutROMString( LCMString2 );

	OpenTimer1( T1_ON & T1_IDLE_CON & T1_GATE_OFF & T1_PS_1_256 & T1_SYNC_EXT_OFF & T1_SOURCE_INT ,
			    Timer1Tick );

	ConfigIntTimer1( T1_INT_ON & T1_INT_PRIOR_4 );

	OpenTimer3( T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_256 & T3_SOURCE_INT , Timer3Tick );

	// wqgzF, (adc.h ̭ٺ| ||defined( adc_v2_1 _ ) ).
	// C:\Program Files\Microchip\MPLAB C30\support\peripheral_24F\adc.h Line 588:
	// /*** AD1PCFGL register definition ***/
	// #if defined (adc_v1_1) || defined (adc_v1_2) || defined (adc_v1_3) || defined (adc_v1_4) || defined (adc_v1_5) "\"
	// || defined (adc_v1_6) || defined (adc_v1_7) || defined (adc_v2_1)

	OpenADC10_v2( ADC_MODULE_ON & ADC_IDLE_CONTINUE & ADC_FORMAT_INTG & ADC_CLK_TMR3 & ADC_AUTO_SAMPLING_ON & ADC_SAMP_OFF ,
   				  ADC_VREF_AVDD_AVSS & ADC_SCAN_OFF & ADC_INTR_EACH_CONV & ADC_ALT_BUF_ON & ADC_ALT_INPUT_OFF ,
				  ADC_SAMPLE_TIME_31 & ADC_CONV_CLK_SYSTEM & ADC_CONV_CLK_64Tcy ,
				  ENABLE_AN5_ANA , 0 , DISABLE_ALL_INPUT_SCAN , 0 );

	SetChanADC10( ADC_CH0_POS_SAMPLEA_AN5 & ADC_CH0_NEG_SAMPLEA_VREFN & ADC_CH0_POS_SAMPLEB_AN5 & ADC_CH0_NEG_SAMPLEB_VREFN ); 

	ConfigIntADC10( ADC_INT_ENABLE & ADC_INT_PRI_4 );

	while( 1 )
	{
		// ഫ ?
		if( ADDown )
		{
			ADDown = 0;

			VR1Value = ( unsigned int ) ReadADC10( 0 );

			LCM_SetCursor( 0 , 1 );
			LCM_PutROMString( LCMString2 );
			LCM_PutNumber( VR1Value , 4 );
		}
	}

	while( 1 );
}
